<?php
/**
*
* @package Partage
* @version 4.0.0
* @copyright (c) 2009 Mathieu Pellegrin
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

/**
* @ignore
*/
if (!defined('IN_PARTAGE')) exit();

/**
* @name Partage
*/
class Partage {

	public $partage_root_absolute;
	private $partage_root_relative;
	private $current_dir_absolute;
	private $current_dir_relative;

	private $include_path;

	private $path_separator;
	private $images_path_name;
	private $images_extension;
	private $file_types;
	private $forbidden_extensions;
	private $forbidden_directories;
	private $never_download;

	private $show_hidden_files;
	private $download_hidden_files;
	private $stop_on_error;
	private $check_input_on_construct;
	private $use_path_info;

	private $logfile;



	/*************/
	/* Accessors */
	/*************/

	public function get_root_url()
	{
			return $this->root_url;
	}

	public function get_root($type = ABSOLUTE_PATH)
	{
		if ($type == ABSOLUTE_PATH)
			return $this->partage_root_absolute;
		else
			return $this->partage_root_relative;
	}

	public function get_current($type = ABSOLUTE_PATH)
	{
		if ($type == ABSOLUTE_PATH)
			return $this->current_dir_absolute;
		else
			return $this->current_dir_relative;
	}

	public function get_path_separator()
	{
		return $this->path_separator;
	}

	public function get_input_methode()
	{
		return ($this->use_path_info ? URL_PATH_INFO : URL_GET);
	}


	/*********************/
	/* Program functions */
	/*********************/

	/**
	* @name __construct
	* @param string $mypath The current working path
	* @return nothing
	* @description Constructor
	* @note
	*/
	public function __construct($mypath = '')
	{
		$this->partage_root_relative = $mypath;
		$this->partage_root_absolute = $this->standardise(realpath($this->partage_root_relative));

		$this->include_path = $this->partage_root_relative.'./prgmdata/'; //A modifier aussi dans config.inc.php et makerss.php
		include_once($mypath.$this->include_path.'config.inc.php');

		if ($this->check_input_on_construct)
		{
			if (!$this->use_path_info)
				$this->check_input($_GET);
			else
			{
				$input = array('path_info' => @$_SERVER['PATH_INFO']);
				$this->check_input($input);
			}
		}
	}


	/**
	* @name error_hack
	* @param string $reason The reason of the stop
	* @return nothing
	* @description Immediately stops the program, display a message, and eventually logs what has happen
	* @note
	*/
	public function error_hack($reason)
	{
		if ($this->stop_on_error)
		{
			echo "Hacking attempt !";
			if (debug)
				echo "<br /><br />".$reason;
			if (log_errors) {
				$handle=fopen($this->logfile,"a");
				fwrite($handle,date("[d-m-Y H:i:s]")." ²²² ".$_SERVER['REMOTE_ADDR']." ²²² ".$_SERVER['QUERY_STRING']." ²²² ".$reason."\r\n");
				fclose($handle);
			}
			exit();
		}
		else
		{
			return $reason;
		}
	}


	/**
	* @name get_extension
	* @param string $type The file name or file full path
	* @return string the relative path of the image
	* @description Gives the file extension
	* @note
	*/
	public function get_extension($string)
	{
		$ext=strrchr($string,'.');
		if ($ext===FALSE)
			$ext='generic';
		else	//Enlève le 1e caractère (".")
			$ext=substr($ext,1,strlen($ext)-1);
		$ext=strtolower($ext);  // Mise en minuscules
		return $ext;
	}

	/**
	* @name get_image_from_type
	* @param string $type The file type
	* @return string the relative path of the image
	* @description Give the image associated to filetype
	* @note
	*/
	public function get_image_from_type($type)
	{
		if ($type=='dir' || $type=='file')
			return $this->get_root_url().$this->images_path . $type . $this->images_extension;
		else
			return;
	}


	/**
	* @name get_image_from_extension
	* @param string $ext The extension
	* @return string the relative path of the image
	* @description Give the image associated to an extension
	* @note
	*/
	public function get_image_from_extension($ext)
	{
		$name = $this->get_root_url().$this->images_path;
		if (@$this->file_types[$ext])
			$name .= $this->file_types[$ext];
		else
			$name .= $this->file_types['unknown'];
		$name .= $this->images_extension;

		return $name;
	}


	/**
	* @name get_image_from_filename
	* @param string $filename The file name
	* @return string the relative path of the image
	* @description Give the image associated to a file name
	* @note
	*/
	public function get_image_from_filename($filename)
	{
		return $this->get_image_from_extension($this->get_extension($filename));
	}


	/**
	* @name cut_root
	* @param string $root If specified alone, this is the path to cut with the absolute root saved in attributes, If the second argument is specified, this is the root to be used.
	* @param string $dir If specified, this is the path where the root has to be removed
	* @return string the new path
	* @description Remove the root path from the given path
	* @note Used in fact to convert the path into a relative path
	*/
	public function cut_root($root,$dir = FALSE)
	{
		if ($dir === FALSE)
		{
			return $this->cut_root($this->partage_root_absolute, $root);
		}
		else
		{
			$lroot=strlen($root);
			return '.'.$this->path_separator.substr($dir,$lroot,strlen($dir)-$lroot);
		}
	}



	/**
	* @name standardise
	* @param string $dir the path work with
	* @return string the path cleaned
	* @description Clean the path to make it as standard as possible, ie /blah/blahblah/blah/
	* @note In most cases, simply add '/' at the end of path
	*/
	public function standardise($dir)
	{
		if (substr($dir,-1)!=$this->path_separator)
			$dir=$dir.$this->path_separator;

		return $dir;
	}


	/**
	* @name make_real
	* @param string $dir the path work with
	* @return string the path cleaned
	* @description Clean the path to make it as realpath
	* @note
	*/
	public function make_real($dir)
	{
		$real=realpath($dir);
		$real=$this->standardise($real);

		return $real;
	}


	/**
	* @name is_forbidden_dir
	* @param string $dir the path to test
	* @return boolean TRUE if the directory listing is forbidden, and FALSE either
	* @description Test if the directory listing is allowed or not
	* @note WARNING: $dir have to be an absolute path !
	*/
	public function is_forbidden_dir($dir)
	{
		/*
		if (substr($dir,-1)!=$this->path_separator)
			$dir=$dir.$this->path_separator;
		*/
		$dir = $this->standardise($dir);
		foreach ($this->forbidden_directories as $value)
		{
			if (substr($dir,0,strlen($value))==$value && $value!="")
				return true;
		}
		return false;
	}


	/**
	* @name is_forbidden_extension
	* @param string $file the file name to test
	* @return boolean TRUE if the extension is forbidden, and FALSE either
	* @description Test if the extension listing is allowed or not
	* @note
	*/
	public function is_forbidden_extension($file)
	{
		return in_array($this->get_extension($file),$this->forbidden_extensions);
	}


	/**
	* @name ls_dir
	* @param string $dir the path we want to list
	* @param int $filter The file type filter
	* @return array The directory listing
	* @description List the given directory, and filter files and directories
	* @note
	*/
	public function ls_dir($dir = NULL , $filter = LIST_ALL, $infos = INFOS_ALL)
	{
		if (! (($filter & LIST_DIRECTORIES) || ($filter & LIST_FILES)) )
			return;
		if (! (($infos & INFOS_PATH) || ($infos & INFOS_NAME) || ($infos & INFOS_TIMESTAMP)) )
			return;

		if ($dir === NULL)
			$dir = $this->get_current(ABSOLUTE_PATH);
		else
		{
			if ($this->is_forbidden_dir($dir))
				return;
		}

		$handle = opendir($dir);

		$iterator = 0;
		$list = array();

		while ($file = readdir($handle))
		{
			if (! (!$this->show_hidden_files && substr($file,0,1)=='.') )
			{
				$type = (is_dir($dir.$this->get_path_separator().$file) ? "dir" : "file");

				if ($type == "dir" && ($filter & LIST_DIRECTORIES))
				{
					$real = $this->make_real($dir.$this->get_path_separator().$file);
					if ($file != "." && $file != ".." && !$this->is_forbidden_dir($real))
					{
						if ($infos & INFOS_PATH)
							$list[$iterator]['path'] = $this->standardise($dir.$file);
						if ($infos & INFOS_NAME)
							$list[$iterator]['name'] = $file;
						if ($infos & INFOS_TIMESTAMP)
							$list[$iterator]['date'] = filemtime($dir.$file);
					}
				}
				if ($type == "file" && ($filter & LIST_FILES))
				{
					$ext = $this->get_extension($file);

					if (!$this->is_forbidden_extension($file))
					{
						if ($infos & INFOS_PATH)
							$list[$iterator]['path'] = $this->standardise($dir).$file;
						if ($infos & INFOS_NAME)
							$list[$iterator]['name'] = $file;
						if ($infos & INFOS_TIMESTAMP)
							$list[$iterator]['date'] = filemtime($dir.$file);
					}
				}
				$iterator++;
			}
		}
		closedir($handle);

		return $list;
	}


	/**
	* @name fetch_all_subdirs
	* @param string $dir starting directory
	* @param char $glue The directories separator
	* @return array the subdirectories path list
	* @description Fetch a list of all subdirectories of a given directory
	* @note Used only by the RSS and Sitemap makers, this is very slow function !
	*/
	private function fetch_all_subdirs($dir) {
		$glue = $this->get_path_separator();
		$adresse = Array();
		$i = 0;
		$handle = opendir($dir);
		rewinddir($handle);
		while ($namefile = readdir($handle)) {
			$founddir = realpath($dir.$glue.$namefile);
			if (substr($founddir,-1)!=$glue)
				$founddir = $founddir.$glue;
			if (is_dir($founddir) && $namefile!="." && $namefile!=".." && !$this->is_forbidden_dir($founddir)) {
				if (!in_array($founddir,$adresse))
					$adresse[$i]=$founddir;
				$i++;
			}
		}
		closedir($handle);

		return @$adresse;
	}


	/**
	* @name make_download_url
	* @param string $fullpath The full path file
	* @param int $format The url format
	* @return string The formated download URL
	* @description Return the formated download URL
	* @note
	*/
	public function make_download_url($fullpath, $format = NULL)
	{
		if ($format===NULL)
			$format = ($this->use_path_info ? URL_PATH_INFO : URL_GET);

		if ($format == URL_PATH_INFO)
			return $this->get_root_url().'index.php/'.str_replace(' ','%20',substr($this->cut_root($this->get_root(),$fullpath),2,strlen($fullpath)));
		if ($format == URL_GET)
			return $this->get_root_url().'index.php?dl='.rawurlencode($this->cut_root($fullpath));
	}


	/**
	* @name make_chdir_url
	* @param string $fullpath The full path
	* @param int $format The url format
	* @return string The formated URL to change the directory
	* @description Return the formated URL used to change the directory
	* @note
	*/
	public function make_chdir_url($fullpath, $format = NULL)
	{
		if ($format===NULL)
			$format = ($this->use_path_info ? URL_PATH_INFO : URL_GET);

		if ($format == URL_PATH_INFO)
			return $this->get_root_url().'index.php/'.str_replace(' ','%20',substr($this->cut_root($this->get_root(),$fullpath),2,strlen($fullpath)));
		if ($format == URL_GET)
			return $this->get_root_url().'index.php?dir='.rawurlencode($this->cut_root($fullpath));
	}


	/*if (!function_exists("scandir"))
	{
		function scandir($dir) {
			global $glue;
			$handle=opendir($dir);
			rewinddir($handle);
			while ($namefile=readdir($handle)) {
				if (is_file($dir.$glue.$namefile)) {
					$files[sizeof(@$files)]=$namefile;
				}
			}
			closedir($handle);
			if (@$files)
				return $files;
			else
				return 0;
		}
	}*/


	/**
	* @name download_file_proxied
	* @param string $file The wanted file
	* @return nothing
	* @description Regular file download
	* @note Could be modified to add a stat counter
	* @note WARNING: Does not check is the download is allowed or not !
	*/
	private function download_file_proxied($file)
	{
		chdir($this->get_root());
		$namefile=strrchr($file,$this->path_separator);
		$namefile=substr($namefile,1,strlen($namefile)-1); 	//Enlève le 1e caractère ("/")
		$namefile=str_replace(" ","_",$namefile);
		header("Content-Length: ".filesize($file));
		if (function_exists("mime_content_type") && mime_content_type($file)!="")
			header("Content-type: ".mime_content_type($file));
		else
			if (isset($this->content_types[$this->get_extension($file)]) && $this->content_types[$this->get_extension($file)]!="")
				header("Content-type: ".$this->content_types[$this->get_extension($file)]);
			else
			{
				header("Content-type: application/octet-stream");
				header("Content-Disposition: attachment; filename=".$namefile);
			}
		readfile($file);
		exit();
	}


	/**
	* @name download_file_unproxied
	* @param string $file The wanted file
	* @return nothing
	* @description Special file download (security issue)
	* @note Prevents a file to be downloaded from readfile by showing it in the browser, however, it also shows his full path
	* @note WARNING: Does not check is the download is allowed or not !
	*/
	private function download_file_unproxied($file)
	{
		//Ben oui, stupidement (pas complètement quand meme)  on a rajouté le $root.$separator, il faut l'enlever maintenant
		//$file = substr($file,strlen($this->partage_root_absolute),strlen($file)-strlen($this->partage_root_absolute));
		do {
			$ok=true;
			if (substr($file,0,2) == '.' . $this->path_separator)
			{
				$file = substr($file,2,strlen($file)-2);
				$ok = false;
			}
			if (substr($file,0,1) == $this->path_separator)
			{
				$file = substr($file,1,strlen($file)-1);
				$ok = false;
			}
		} while(!$ok);
		//Windows qui fait des siennes...
		if ($this->path_separator == "\\")
			$file = str_replace("\\","/",$file);
		header('Location: ./'.$file);
		exit();
	}


	/**
	* @name check_input
	* @param array $input the input array
	* @return The error array
	* @description Check the input array given
	* @note
	*/
	public function check_input($input)
	{
		$errors = array();

		if (!$this->use_path_info)
		{
			$this->current_dir_absolute = $this->partage_root_absolute;
			if (@$input['dir'])
			{
				if (file_exists(realpath($this->partage_root_absolute.$input['dir'])))
					$this->current_dir_absolute = realpath($this->partage_root_absolute.$input['dir']);
				else
					$errors[] = $this->error_hack("Chemin de dossier corrompu: dossier introuvable");
			}
			$this->current_dir_absolute = $this->standardise($this->current_dir_absolute);

			if (@$input['dir'])
			{
				if ($this->is_forbidden_dir($this->current_dir_absolute))
					$errors[] = $this->error_hack("Chemin de dossier corrompu: dossier caché et interdit");
				if (!is_dir($this->current_dir_absolute))
					$errors[] = $this->error_hack("Chemin de dossier corrompu: ce n'est pas un dossier");
			}

			if (substr($this->current_dir_absolute,0,strlen($this->partage_root_absolute))!=$this->partage_root_absolute)
				$errors[] = $this->error_hack("Chemin de dossier corrompu: tentative de déplacement au dessus de la racine");

			if (@$input['dl'])
			{
				if (!$this->download_hidden_files && substr($input['dl'],strrpos($input['dl'],$this->path_separator)+1,1)=='.')
					$errors[] = $this->error_hack("Chemin de fichier corrompu: fichier caché interdit au téléchargement");

				if (strpos($input['dl'],$this->path_separator)===FALSE)
					$errors[] = $this->error_hack("Chemin de fichier corrompu: absence de séparateur de dossiers valide");

				$input['dl'] = $this->partage_root_absolute.$this->path_separator.$input['dl'];
				$pathdl=realpath(dirname($input['dl']));

				$pathdl=$this->standardise($pathdl);

				if (substr($pathdl,0,strlen($this->partage_root_absolute))!=$this->partage_root_absolute || !file_exists($input['dl']) || !is_file($input['dl']))
					$errors[] = $this->error_hack("Chemin de fichier corrompu: système de fichier attaqué");
				if ($this->is_forbidden_extension($input['dl']) || $this->is_forbidden_dir($pathdl))
					$errors[] = $this->error_hack("Chemin de fichier corrompu: fichier caché et interdit");
			}

			$this->current_dir_relative = $this->standardise(".".$this->path_separator.substr($this->current_dir_absolute,strlen($this->partage_root_absolute),strlen($this->current_dir_absolute)-strlen($this->partage_root_absolute)));

		}
		else
		{

			$this->current_dir_absolute = $this->partage_root_absolute;

			$input['path_info'] = $this->current_dir_absolute.$this->path_separator.$input['path_info'];

			if (file_exists($input['path_info']))
			{
				if (is_dir($input['path_info']))
				{
					if (file_exists(realpath($input['path_info'])))
						$this->current_dir_absolute = realpath($input['path_info']);
					else
						$errors[] = $this->error_hack("Chemin de dossier corrompu: dossier introuvable");
				}
				$this->current_dir_absolute = $this->standardise($this->current_dir_absolute);

				if (is_dir($input['path_info']))
				{
					if ($this->is_forbidden_dir($this->current_dir_absolute))
						$errors[] = $this->error_hack("Chemin de dossier corrompu: dossier caché et interdit");
					if (!is_dir($this->current_dir_absolute))
						$errors[] = $this->error_hack("Chemin de dossier corrompu: ce n'est pas un dossier");
				}

				if (substr($this->current_dir_absolute,0,strlen($this->partage_root_absolute))!=$this->partage_root_absolute)
					$errors[] = $this->error_hack("Chemin de dossier corrompu: tentative de déplacement au dessus de la racine");

				if (is_file($input['path_info']))
				{
					if (!$this->download_hidden_files && substr($input['path_info'],strrpos($input['path_info'],$this->path_separator)+1,1)=='.')
						$errors[] = $this->error_hack("Chemin de fichier corrompu: fichier caché interdit au téléchargement");

					if (strpos($input['path_info'],$this->path_separator)===FALSE)
						$errors[] = $this->error_hack("Chemin de fichier corrompu: absence de séparateur de dossiers valide");

					$pathdl = realpath(dirname($input['path_info']));
					$pathdl = $this->standardise($pathdl);

					if (substr($pathdl,0,strlen($this->partage_root_absolute))!=$this->partage_root_absolute || !file_exists($input['path_info']) || !is_file($input['path_info']))
						$errors[] = $this->error_hack("Chemin de fichier corrompu: système de fichier attaqué");
					if ($this->is_forbidden_extension($input['path_info']) || $this->is_forbidden_dir($pathdl))
						$errors[] = $this->error_hack("Chemin de fichier corrompu: fichier caché et interdit");
				}

				$this->current_dir_relative = $this->standardise(".".$this->path_separator.substr($this->current_dir_absolute,strlen($this->partage_root_absolute),strlen($this->current_dir_absolute)-strlen($this->partage_root_absolute)));
			}
			else
				$errors[] = $this->error_hack("Chemin de dossier corrompu: fichier ou dossier introuvable");

		}
		return $errors;
	}


	/**
	* @name download_file
	* @param string $file The wanted file
	* @return nothing
	* @description Simply starts the download
	* @note
	*/
	public function download_file($file)
	{
		$path = realpath(substr($file,0,strrpos($this->path_separator,$file)));

		if (!$this->is_forbidden_dir($path) && !$this->is_forbidden_extension($file))
		{
			if ($file!="")
			{
				if (in_array($this->get_extension($file), $this->never_download))
					$this->download_file_unproxied($file);
				else
					$this->download_file_proxied($file);
				//Pour être sûr de pas continuer, et donner le fichier
				exit();
			}
		}
		else
		{
			$this->error_hack('Téléchargement corrompu.');
		}
	}

}
?>
